struct User {
    active: bool,
    username: String, // using &str wont compile
    email: String,    // needs a lifetime which & doesn't do
    sign_in_count: u64,
}

// Just like haskell :)
#[derive(Debug)]
struct Color(i32, i32, i32);
#[derive(Debug)]
struct Point(i32, i32, i32);

struct AlwaysEqual;

fn main() {
    let mut user1 = User {
        email: String::from("email@mail.org"),
        username: String::from("username"),
        active: true,
        sign_in_count: 1,
    };
    user1.email = String::from("new_email@mail.org");

    // update syntax
    // borrows from user1, deallocating only the copied variables in the process.
    // The variables not borrowed are left alone. Rust is cool
    let user2 = User {
        // active: user1.active,
        // username: user1.username,
        email: String::from("other_email@mail.org"),
        // sign_in_count: user1.sign_in_count,
        ..user1 // can do this for copying fields over on the shorthand
    };

    let black = Color(0, 0, 0);
    let origin = Point(0, 0, 0);

    println!("black: {:?}", black);
    println!("origin: {:?}", origin);

    // unit-like structs without any fields
    let subject = AlwaysEqual;
}

fn build_user(email: String, username: String) -> User {
    User {
        active: true,
        username, // use a shorthand because this errors if not using a shorthand?
        email,    // That might have just been linting
        sign_in_count: 1,
    }
}
